@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2015 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011-2016 -- leonerd@leonerd.org.uk
 
 package Future;
 
@@ -9,7 +9,7 @@ use strict;
 use warnings;
 no warnings 'recursion'; # Disable the "deep recursion" warning
 
-our $VERSION = '0.32';
+our $VERSION = '0.35';
 
 use Carp qw(); # don't import croak
 use Scalar::Util qw( weaken blessed reftype );
@@ -76,6 +76,34 @@ involved in such situations.
 See also L<Future::Utils> which contains useful loop-constructing functions,
 to run a future-returning function repeatedly in a loop.
 
+Unless otherwise noted, the following methods require at least version
+I<0.08>.
+
+=head2 FAILURE CATEGORIES
+
+While not directly required by C<Future> or its related modules, a growing
+convention of C<Future>-using code is to encode extra semantics in the
+arguments given to the C<fail> method, to represent different kinds of
+failure.
+
+The convention is that after the initial message string as the first required
+argument (intended for display to humans), the second argument is a short
+lowercase string that relates in some way to the kind of failure that
+occurred. Following this is a list of details about that kind of failure,
+whose exact arrangement or structure are determined by the failure category.
+For example, L<IO::Async> and L<Net::Async::HTTP> use this convention to
+indicate at what stage a given HTTP request has failed:
+
+   ->fail( $message, http => ... )  # an HTTP-level error during protocol
+   ->fail( $message, connect => ... )  # a TCP-level failure to connect a
+                                       # socket
+   ->fail( $message, resolve => ... )  # a resolver (likely DNS) failure
+                                       # to resolve a hostname
+
+By following this convention, a module remains consistent with other
+C<Future>-based modules, and makes it easy for program logic to gracefully
+handle and manage failures by use of the C<catch> method.
+
 =head2 SUBCLASSING
 
 This class easily supports being subclassed to provide extra behavior, such as
@@ -116,21 +144,6 @@ the function or method calls.
 The F<examples> directory in the distribution contains some examples of how
 futures might be integrated with various event systems.
 
-=head2 MODULE DOCUMENTATION
-
-Modules that provide future-returning functions or methods may wish to adopt
-the following styles in some way, to document the eventual return values from
-these futures.
-
- func( ARGS, HERE... ) ==> ( RETURN, VALUES... )
-
- OBJ->method( ARGS, HERE... ) ==> ( RETURN, VALUES... )
-
-Code returning a future that yields no values on success can use empty
-parentheses.
-
- func( ... ) ==> ()
-
 =head2 DEBUGGING
 
 By the time a C<Future> object is destroyed, it ought to have been completed
@@ -138,7 +151,6 @@ or cancelled. By enabling debug tracing of objects, this fact can be checked.
 If a future object is destroyed without having been completed or cancelled, a
 warning message is printed.
 
-
  $ PERL_FUTURE_DEBUG=1 perl -MFuture -E 'my $f = Future->new'
  Future=HASH(0xaa61f8) was constructed at -e line 1 and was lost near -e line 0 before it was ready.
 
@@ -177,9 +189,11 @@ converging method).
 
 =cut
 
-=head2 $future = Future->new
+=head2 new
 
-=head2 $future = $orig->new
+   $future = Future->new
+
+   $future = $orig->new
 
 Returns a new C<Future> instance to represent a leaf future. It will be marked
 as ready by any of the C<done>, C<fail>, or C<cancel> methods. It can be
@@ -275,14 +289,24 @@ sub DESTROY_debug {
 }
 *DESTROY = \&DESTROY_debug if DEBUG;
 
-=head2 $future = Future->done( @values )
+=head2 done I<(class method)>
 
-=head2 $future = Future->fail( $exception, @details )
+=head2 fail I<(class method)>
+
+   $future = Future->done( @values )
+
+   $future = Future->fail( $exception, @details )
+
+I<Since version 0.26.>
 
 Shortcut wrappers around creating a new C<Future> then immediately marking it
 as done or failed.
 
-=head2 $future = Future->wrap( @values )
+=head2 wrap
+
+   $future = Future->wrap( @values )
+
+I<Since version 0.14.>
 
 If given a single argument which is already a C<Future> reference, this will
 be returned unmodified. Otherwise, returns a new C<Future> instance that is
@@ -307,7 +331,11 @@ sub wrap
    }
 }
 
-=head2 $future = Future->call( \&code, @args )
+=head2 call
+
+   $future = Future->call( \&code, @args )
+
+I<Since version 0.15.>
 
 A convenient wrapper for calling a C<CODE> reference that is expected to
 return a future. In normal circumstances is equivalent to
@@ -455,7 +483,9 @@ interfaces.
 
 =cut
 
-=head2 $future->done( @result )
+=head2 done
+
+   $future->done( @result )
 
 Marks that the leaf future is now ready, and provides a list of values as a
 result. (The empty list is allowed, and still indicates the future as ready).
@@ -464,15 +494,6 @@ Cannot be called on a convergent future.
 If the future is already cancelled, this request is ignored. If the future is
 already complete with a result or a failure, an exception is thrown.
 
-=head2 Future->done( @result )
-
-May also be called as a class method, where it will construct a new Future and
-immediately mark it as done.
-
-Returns the C<$future> to allow easy chaining to create an immediate future by
-
- return Future->done( ... )
-
 =cut
 
 sub done
@@ -496,25 +517,15 @@ sub done
    return $self;
 }
 
-=head2 $code = $future->done_cb
-
-Returns a C<CODE> reference that, when invoked, calls the C<done> method. This
-makes it simple to pass as a callback function to other code.
-
-As the same effect can be achieved using L<curry>, this method is deprecated
-now and may be removed in a later version.
-
- $code = $future->curry::done;
-
-=cut
-
 sub done_cb
 {
    my $self = shift;
    return sub { $self->done( @_ ) };
 }
 
-=head2 $future->fail( $exception, @details )
+=head2 fail
+
+   $future->fail( $exception, @details )
 
 Marks that the leaf future has failed, and provides an exception value. This
 exception will be thrown by the C<get> method if called. 
@@ -527,16 +538,6 @@ by C<get>.
 If the future is already cancelled, this request is ignored. If the future is
 already complete with a result or a failure, an exception is thrown.
 
-=head2 Future->fail( $exception, @details )
-
-May also be called as a class method, where it will construct a new Future and
-immediately mark it as failed.
-
-Returns the C<$future> to allow easy chaining to create an immediate failed
-future by
-
- return Future->fail( ... )
-
 =cut
 
 sub fail
@@ -569,25 +570,17 @@ sub fail
    return $self;
 }
 
-=head2 $code = $future->fail_cb
-
-Returns a C<CODE> reference that, when invoked, calls the C<fail> method. This
-makes it simple to pass as a callback function to other code.
-
-As the same effect can be achieved using L<curry>, this method is deprecated
-now and may be removed in a later version.
-
- $code = $future->curry::fail;
-
-=cut
-
 sub fail_cb
 {
    my $self = shift;
    return sub { $self->fail( @_ ) };
 }
 
-=head2 $future->die( $message, @details )
+=head2 die
+
+   $future->die( $message, @details )
+
+I<Since version 0.09.>
 
 A convenient wrapper around C<fail>. If the exception is a non-reference that
 does not end in a linefeed, its value will be extended by the file and line
@@ -609,7 +602,9 @@ sub die :method
    $self->fail( $exception, @details );
 }
 
-=head2 $future->on_cancel( $code )
+=head2 on_cancel
+
+   $future->on_cancel( $code )
 
 If the future is not yet ready, adds a callback to be invoked if the future is
 cancelled by the C<cancel> method. If the future is already ready, throws an
@@ -620,8 +615,6 @@ to that in which they were registered.
 
  $on_cancel->( $future )
 
-=head2 $future->on_cancel( $f )
-
 If passed another C<Future> instance, the passed instance will be cancelled
 when the original future is cancelled. This method does nothing if the future
 is already complete.
@@ -644,7 +637,9 @@ sub on_cancel
    return $self;
 }
 
-=head2 $cancelled = $future->is_cancelled
+=head2 is_cancelled
+
+   $cancelled = $future->is_cancelled
 
 Returns true if the future has been cancelled by C<cancel>.
 
@@ -663,7 +658,9 @@ objects returned by such an interface.
 
 =cut
 
-=head2 $ready = $future->is_ready
+=head2 is_ready
+
+   $ready = $future->is_ready
 
 Returns true on a leaf future if a result has been provided to the C<done>
 method, failed using the C<fail> method, or cancelled using the C<cancel>
@@ -680,7 +677,9 @@ sub is_ready
    return $self->{ready};
 }
 
-=head2 $future->on_ready( $code )
+=head2 on_ready
+
+   $future->on_ready( $code )
 
 If the future is not yet ready, adds a callback to be invoked when the future
 is ready. If the future is already ready, invokes it immediately.
@@ -690,14 +689,12 @@ invoked code can then obtain the list of results by calling the C<get> method.
 
  $on_ready->( $future )
 
-Returns the C<$future>.
-
-=head2 $future->on_ready( $f )
-
 If passed another C<Future> instance, the passed instance will have its
 C<done>, C<fail> or C<cancel> methods invoked when the original future
 completes successfully, fails, or is cancelled respectively.
 
+Returns the C<$future>.
+
 =cut
 
 sub on_ready
@@ -727,7 +724,9 @@ sub on_ready
    return $self;
 }
 
-=head2 $done = $future->is_done
+=head2 is_done
+
+   $done = $future->is_done
 
 Returns true on a future if it is ready and completed successfully. Returns
 false if it is still pending, failed, or was cancelled.
@@ -740,9 +739,11 @@ sub is_done
    return $self->{ready} && !$self->{failure} && !$self->{cancelled};
 }
 
-=head2 @result = $future->get
+=head2 get
 
-=head2 $result = $future->get
+   @result = $future->get
+
+   $result = $future->get
 
 If the future is ready and completed successfully, returns the list of
 results that had earlier been given to the C<done> method on a leaf future,
@@ -770,7 +771,7 @@ sub await
 sub get
 {
    my $self = shift;
-   $self->await until $self->{ready};
+   until( $self->{ready} ) { $self->await }
    if( $self->{failure} ) {
       $self->{reported} = 1;
       my $exception = $self->{failure}->[0];
@@ -781,7 +782,11 @@ sub get
    return @{ $self->{result} };
 }
 
-=head2 @values = Future->unwrap( @values )
+=head2 unwrap
+
+   @values = Future->unwrap( @values )
+
+I<Since version 0.26.>
 
 If given a single argument which is a C<Future> reference, this method will
 call C<get> on it and return the result. Otherwise, it returns the list of
@@ -809,7 +814,9 @@ sub unwrap
    }
 }
 
-=head2 $future->on_done( $code )
+=head2 on_done
+
+   $future->on_done( $code )
 
 If the future is not yet ready, adds a callback to be invoked when the future
 is ready, if it completes successfully. If the future completed successfully,
@@ -820,13 +827,11 @@ The callback will be passed the result passed to the C<done> method.
 
  $on_done->( @result )
 
-Returns the C<$future>.
-
-=head2 $future->on_done( $f )
-
 If passed another C<Future> instance, the passed instance will have its
 C<done> method invoked when the original future completes successfully.
 
+Returns the C<$future>.
+
 =cut
 
 sub on_done
@@ -851,7 +856,11 @@ sub on_done
    return $self;
 }
 
-=head2 $failed = $future->is_failed
+=head2 is_failed
+
+   $failed = $future->is_failed
+
+I<Since version 0.26.>
 
 Returns true on a future if it is ready and it failed. Returns false if it is
 still pending, completed successfully, or was cancelled.
@@ -864,9 +873,11 @@ sub is_failed
    return $self->{ready} && !!$self->{failure}; # boolify
 }
 
-=head2 $exception = $future->failure
+=head2 failure
 
-=head2 $exception, @details = $future->failure
+   $exception = $future->failure
+
+   $exception, @details = $future->failure
 
 Returns the exception passed to the C<fail> method, C<undef> if the future
 completed successfully via the C<done> method, or raises an exception if
@@ -891,14 +902,16 @@ statement:
 sub failure
 {
    my $self = shift;
-   $self->await until $self->{ready};
+   until( $self->{ready} ) { $self->await }
    return unless $self->{failure};
    $self->{reported} = 1;
    return $self->{failure}->[0] if !wantarray;
    return @{ $self->{failure} };
 }
 
-=head2 $future->on_fail( $code )
+=head2 on_fail
+
+   $future->on_fail( $code )
 
 If the future is not yet ready, adds a callback to be invoked when the future
 is ready, if it fails. If the future has already failed, invokes it
@@ -910,10 +923,6 @@ method.
 
  $on_fail->( $exception, @details )
 
-Returns the C<$future>.
-
-=head2 $future->on_fail( $f )
-
 If passed another C<Future> instance, the passed instance will have its
 C<fail> method invoked when the original future fails.
 
@@ -922,6 +931,8 @@ reference:
 
  $future->on_fail( sub { $f->done( @_ ) } );
 
+Returns the C<$future>.
+
 =cut
 
 sub on_fail
@@ -947,7 +958,9 @@ sub on_fail
    return $self;
 }
 
-=head2 $future->cancel
+=head2 cancel
+
+   $future->cancel
 
 Requests that the future be cancelled, immediately marking it as ready. This
 will invoke all of the code blocks registered by C<on_cancel>, in the reverse
@@ -976,18 +989,6 @@ sub cancel
    return $self;
 }
 
-=head2 $code = $future->cancel_cb
-
-Returns a C<CODE> reference that, when invoked, calls the C<cancel> method.
-This makes it simple to pass as a callback function to other code.
-
-As the same effect can be achieved using L<curry>, this method is deprecated
-now and may be removed in a later version.
-
- $code = $future->curry::cancel;
-
-=cut
-
 sub cancel_cb
 {
    my $self = shift;
@@ -1077,7 +1078,11 @@ sub _sequence
    return $fseq;
 }
 
-=head2 $future = $f1->then( \&done_code )
+=head2 then
+
+   $future = $f1->then( \&done_code )
+
+I<Since version 0.13.>
 
 Returns a new sequencing C<Future> that runs the code if the first succeeds.
 Once C<$f1> succeeds the code reference will be invoked and is passed the list
@@ -1088,7 +1093,11 @@ same failure and the code will not be invoked.
 
  $f2 = $done_code->( @result )
 
-=head2 $future = $f1->else( \&fail_code )
+=head2 else
+
+   $future = $f1->else( \&fail_code )
+
+I<Since version 0.13.>
 
 Returns a new sequencing C<Future> that runs the code if the first fails. Once
 C<$f1> fails the code reference will be invoked and is passed the failure and
@@ -1099,7 +1108,9 @@ same result and the code will not be invoked.
 
  $f2 = $fail_code->( $exception, @details )
 
-=head2 $future = $f1->then( \&done_code, \&fail_code )
+=head2 then I<(2 arguments)>
+
+   $future = $f1->then( \&done_code, \&fail_code )
 
 The C<then> method can also be passed the C<$fail_code> block as well, giving
 a combination of C<then> and C<else> behaviour.
@@ -1109,32 +1120,56 @@ systems, such as Javascript's Q or Promises/A libraries.
 
 =cut
 
+my $make_donecatchfail_sub = sub {
+   my ( $with_f, $done_code, $fail_code, @catch_list ) = @_;
+
+   my $func = (caller 1)[3];
+   $func =~ s/^.*:://;
+
+   !$done_code or _callable( $done_code ) or
+      Carp::croak "Expected \$done_code to be callable in ->$func";
+   !$fail_code or _callable( $fail_code ) or
+      Carp::croak "Expected \$fail_code to be callable in ->$func";
+
+   my %catch_handlers = @catch_list;
+   _callable( $catch_handlers{$_} ) or
+      Carp::croak "Expected catch handler for '$_' to be callable in ->$func"
+      for keys %catch_handlers;
+
+   sub {
+      my $self = shift;
+      my @maybe_self = $with_f ? ( $self ) : ();
+
+      if( !$self->{failure} ) {
+         return $self unless $done_code;
+         return $done_code->( @maybe_self, $self->get );
+      }
+      else {
+         my $name = $self->{failure}[1];
+         if( defined $name and $catch_handlers{$name} ) {
+            return $catch_handlers{$name}->( @maybe_self, $self->failure );
+         }
+         return $self unless $fail_code;
+         return $fail_code->( @maybe_self, $self->failure );
+      }
+   };
+};
+
 sub then
 {
    my $self = shift;
-   my ( $done_code, $fail_code ) = @_;
+   my $done_code = shift;
+   my $fail_code = ( @_ % 2 ) ? pop : undef;
+   my @catch_list = @_;
 
-   if( $done_code and !$fail_code ) {
+   if( $done_code and !@catch_list and !$fail_code ) {
       return $self->_sequence( $done_code, CB_SEQ_ONDONE|CB_RESULT );
    }
 
-   !$done_code or _callable( $done_code ) or
-      Carp::croak "Expected \$done_code to be callable in ->then";
-   !$fail_code or _callable( $fail_code ) or
-      Carp::croak "Expected \$fail_code to be callable in ->then";
-
    # Complex
-   return $self->_sequence( sub {
-      my $self = shift;
-      if( !$self->{failure} ) {
-         return $self unless $done_code;
-         return $done_code->( $self->get );
-      }
-      else {
-         return $self unless $fail_code;
-         return $fail_code->( $self->failure );
-      }
-   }, CB_SEQ_ONDONE|CB_SEQ_ONFAIL|CB_SELF );
+   return $self->_sequence( $make_donecatchfail_sub->(
+      0, $done_code, $fail_code, @catch_list,
+   ), CB_SEQ_ONDONE|CB_SEQ_ONFAIL|CB_SELF );
 }
 
 sub else
@@ -1145,7 +1180,71 @@ sub else
    return $self->_sequence( $fail_code, CB_SEQ_ONFAIL|CB_RESULT );
 }
 
-=head2 $future = $f1->transform( %args )
+=head2 catch
+
+   $future = $f1->catch(
+      name => \&code,
+      name => \&code, ...
+   )
+
+I<Since version 0.33.>
+
+Returns a new sequencing C<Future> that behaves like an C<else> call which
+dispatches to a choice of several alternative handling functions depending on
+the kind of failure that occurred. If C<$f1> fails with a category name (i.e.
+the second argument to the C<fail> call) which exactly matches one of the
+string names given, then the corresponding code is invoked, being passed the
+same arguments as a plain C<else> call would take, and is expected to return a
+C<Future> in the same way.
+
+ $f2 = $code->( $exception, $name, @other_details )
+
+If C<$f1> does not fail, fails without a category name at all, or fails with a
+category name that does not match any given to the C<catch> method, then the
+returned sequence future immediately completes with the same result, and no
+block of code is invoked.
+
+If passed an odd-sized list, the final argument gives a function to invoke on
+failure if no other handler matches.
+
+   $future = $f1->catch(
+      name => \&code, ...
+      \&fail_code,
+   )
+
+This feature is currently still a work-in-progress. It currently can only cope
+with category names that are literal strings, which are all distinct. A later
+version may define other kinds of match (e.g. regexp), may specify some sort
+of ordering on the arguments, or any of several other semantic extensions. For
+more detail on the ongoing design, see
+L<https://rt.cpan.org/Ticket/Display.html?id=103545>.
+
+=head2 then I<(multiple arguments)>
+
+   $future = $f1->then( \&done_code, @catch_list, \&fail_code )
+
+I<Since version 0.33.>
+
+The C<then> method can be passed an even-sized list inbetween the
+C<$done_code> and the C<$fail_code>, with the same meaning as the C<catch>
+method.
+
+=cut
+
+sub catch
+{
+   my $self = shift;
+   my $fail_code = ( @_ % 2 ) ? pop : undef;
+   my @catch_list = @_;
+
+   return $self->_sequence( $make_donecatchfail_sub->(
+      0, undef, $fail_code, @catch_list,
+   ), CB_SEQ_ONDONE|CB_SEQ_ONFAIL|CB_SELF );
+}
+
+=head2 transform
+
+   $future = $f1->transform( %args )
 
 Returns a new sequencing C<Future> that wraps the one given as C<$f1>. With no
 arguments this will be a trivial wrapper; C<$future> will complete or fail
@@ -1196,13 +1295,18 @@ sub transform
    }, CB_SEQ_ONDONE|CB_SEQ_ONFAIL|CB_SELF );
 }
 
-=head2 $future = $f1->then_with_f( \&code )
+=head2 then_with_f
 
-Returns a new sequencing C<Future> that runs the code if the first succeeds.
-Identical to C<then>, except that the code reference will be passed both the
-original future, C<$f1>, and its result.
+   $future = $f1->then_with_f( ... )
 
- $f2 = $code->( $f1, @result )
+I<Since version 0.21.>
+
+Returns a new sequencing C<Future> that behaves like C<then>, but also passes
+the original future, C<$f1>, to any functions it invokes.
+
+ $f2 = $done_code->( $f1, @result )
+ $f2 = $catch_code->( $f1, $name, @other_details )
+ $f2 = $fail_code->( $f1, @details )
 
 This is useful for conditional execution cases where the code block may just
 return the same result of the original future. In this case it is more
@@ -1213,14 +1317,28 @@ efficient to return the original future itself.
 sub then_with_f
 {
    my $self = shift;
-   my ( $done_code ) = @_;
+   my $done_code = shift;
+   my $fail_code = ( @_ % 2 ) ? pop : undef;
+   my @catch_list = @_;
 
-   return $self->_sequence( $done_code, CB_SEQ_ONDONE|CB_SELF|CB_RESULT );
+   if( $done_code and !@catch_list and !$fail_code ) {
+      return $self->_sequence( $done_code, CB_SEQ_ONDONE|CB_SELF|CB_RESULT );
+   }
+
+   return $self->_sequence( $make_donecatchfail_sub->(
+      1, $done_code, $fail_code, @catch_list,
+   ), CB_SEQ_ONDONE|CB_SEQ_ONFAIL|CB_SELF );
 }
 
-=head2 $future = $f->then_done( @result )
+=head2 then_done
 
-=head2 $future = $f->then_fail( $exception, @details )
+=head2 then_fail
+
+   $future = $f->then_done( @result )
+
+   $future = $f->then_fail( $exception, @details )
+
+I<Since version 0.22.>
 
 Convenient shortcuts to returning an immediate future from a C<then> block,
 when the result is already known.
@@ -1241,7 +1359,11 @@ sub then_fail
    return $self->_sequence( \@failure, CB_SEQ_ONDONE|CB_SEQ_IMFAIL );
 }
 
-=head2 $future = $f1->else_with_f( \&code )
+=head2 else_with_f
+
+   $future = $f1->else_with_f( \&code )
+
+I<Since version 0.21.>
 
 Returns a new sequencing C<Future> that runs the code if the first fails.
 Identical to C<else>, except that the code reference will be passed both the
@@ -1263,9 +1385,15 @@ sub else_with_f
    return $self->_sequence( $fail_code, CB_SEQ_ONFAIL|CB_SELF|CB_RESULT );
 }
 
-=head2 $future = $f->else_done( @result )
+=head2 else_done
 
-=head2 $future = $f->else_fail( $exception, @details )
+=head2 else_fail
+
+   $future = $f->else_done( @result )
+
+   $future = $f->else_fail( $exception, @details )
+
+I<Since version 0.22.>
 
 Convenient shortcuts to returning an immediate future from a C<else> block,
 when the result is already known.
@@ -1286,7 +1414,31 @@ sub else_fail
    return $self->_sequence( \@failure, CB_SEQ_ONFAIL|CB_SEQ_IMFAIL );
 }
 
-=head2 $future = $f1->followed_by( \&code )
+=head2 catch_with_f
+
+   $future = $f1->catch_with_f( ... )
+
+I<Since version 0.33.>
+
+Returns a new sequencing C<Future> that behaves like C<catch>, but also passes
+the original future, C<$f1>, to any functions it invokes.
+
+=cut
+
+sub catch_with_f
+{
+   my $self = shift;
+   my $fail_code = ( @_ % 2 ) ? pop : undef;
+   my @catch_list = @_;
+
+   return $self->_sequence( $make_donecatchfail_sub->(
+      1, undef, $fail_code, @catch_list,
+   ), CB_SEQ_ONDONE|CB_SEQ_ONFAIL|CB_SELF );
+}
+
+=head2 followed_by
+
+   $future = $f1->followed_by( \&code )
 
 Returns a new sequencing C<Future> that runs the code regardless of success or
 failure. Once C<$f1> is ready the code reference will be invoked and is passed
@@ -1306,17 +1458,11 @@ sub followed_by
    return $self->_sequence( $code, CB_SEQ_ONDONE|CB_SEQ_ONFAIL|CB_SELF );
 }
 
-sub and_then
-{
-   Carp::croak "Future->and_then is now removed; use ->then_with_f instead";
-}
+=head2 without_cancel
 
-sub or_else
-{
-   Carp::croak "Future->or_else is now removed; use ->else_with_f instead";
-}
+   $future = $f1->without_cancel
 
-=head2 $future = $f1->without_cancel
+I<Since version 0.30.>
 
 Returns a new sequencing C<Future> that will complete with the success or
 failure of the original future, but if cancelled, will not cancel the
@@ -1383,11 +1529,13 @@ sub _new_convergent
    return $self;
 }
 
-=head2 $future = Future->wait_all( @subfutures )
+=head2 wait_all
+
+   $future = Future->wait_all( @subfutures )
 
 Returns a new C<Future> instance that will indicate it is ready once all of
 the sub future objects given to it indicate that they are ready, either by
-success, failure or cancellation. Its result will a list of its component
+success, failure or cancellation. Its result will be a list of its component
 futures.
 
 When given an empty list this constructor returns a new immediately-done
@@ -1438,7 +1586,9 @@ sub wait_all
    return $self;
 }
 
-=head2 $future = Future->wait_any( @subfutures )
+=head2 wait_any
+
+   $future = Future->wait_any( @subfutures )
 
 Returns a new C<Future> instance that will indicate it is ready once any of
 the sub future objects given to it indicate that they are ready, either by
@@ -1524,7 +1674,9 @@ sub wait_any
    return $self;
 }
 
-=head2 $future = Future->needs_all( @subfutures )
+=head2 needs_all
+
+   $future = Future->needs_all( @subfutures )
 
 Returns a new C<Future> instance that will indicate it is ready once all of the
 sub future objects given to it indicate that they have completed successfully,
@@ -1619,7 +1771,9 @@ sub needs_all
    return $self;
 }
 
-=head2 $future = Future->needs_any( @subfutures )
+=head2 needs_any
+
+   $future = Future->needs_any( @subfutures )
 
 Returns a new C<Future> instance that will indicate it is ready once any of
 the sub future objects given to it indicate that they have completed
@@ -1730,15 +1884,25 @@ the component futures stored by it.
 
 =cut
 
-=head2 @f = $future->pending_futures
+=head2 pending_futures
 
-=head2 @f = $future->ready_futures
+   @f = $future->pending_futures
 
-=head2 @f = $future->done_futures
+=head2 ready_futures
 
-=head2 @f = $future->failed_futures
+   @f = $future->ready_futures
 
-=head2 @f = $future->cancelled_futures
+=head2 done_futures
+
+   @f = $future->done_futures
+
+=head2 failed_futures
+
+   @f = $future->failed_futures
+
+=head2 cancelled_futures
+
+   @f = $future->cancelled_futures
 
 Return a list of all the pending, ready, done, failed, or cancelled
 component futures. In scalar context, each will yield the number of such
@@ -1783,9 +1947,15 @@ sub cancelled_futures
 
 =head1 TRACING METHODS
 
-=head2 $future = $future->set_label( $label )
+=head2 set_label
 
-=head2 $label = $future->label
+=head2 label
+
+   $future = $future->set_label( $label )
+
+   $label = $future->label
+
+I<Since version 0.28.>
 
 Chaining mutator and accessor for the label of the C<Future>. This should be a
 plain string value, whose value will be stored by the future instance for use
@@ -1813,9 +1983,15 @@ sub __selfstr
    return "$self (\"$self->{label}\")";
 }
 
-=head2 [ $sec, $usec ] = $future->btime
+=head2 btime
 
-=head2 [ $sec, $usec ] = $future->rtime
+=head2 rtime
+
+   [ $sec, $usec ] = $future->btime
+
+   [ $sec, $usec ] = $future->rtime
+
+I<Since version 0.28.>
 
 Accessors that return the tracing timestamps from the instance. These give the
 time the instance was contructed ("birth" time, C<btime>) and the time the
@@ -1842,7 +2018,11 @@ sub rtime
    return $self->{rtime};
 }
 
-=head2 $sec = $future->elapsed
+=head2 elapsed
+
+   $sec = $future->elapsed
+
+I<Since version 0.28.>
 
 If both tracing timestamps are defined, returns the number of seconds of
 elapsed time between them as a floating-point number. If not, returns
@@ -1857,7 +2037,9 @@ sub elapsed
    return $self->{elapsed} ||= tv_interval( $self->{btime}, $self->{rtime} );
 }
 
-=head2 $cb = $future->wrap_cb( $operation_name, $cb )
+=head2 wrap_cb
+
+   $cb = $future->wrap_cb( $operation_name, $cb )
 
 I<Since version 0.31.>
 
@@ -1949,14 +2131,15 @@ indicate when it is complete.
  }
 
 In most cases, the C<done> method will simply be invoked with the entire
-result list as its arguments. In that case, it is simpler to use the
-C<done_cb> wrapper method to create the C<CODE> reference.
+result list as its arguments. In that case, it is convenient to use the
+L<curry> module to form a C<CODE> reference that would invoke the C<done>
+method.
 
     my $future = Future->new;
 
     do_something_async(
        foo => $args{foo},
-       on_done => $future->done_cb,
+       on_done => $future->curry::done,
     );
 
 The caller may then use this future to wait for a result using the C<on_ready>
@@ -2171,6 +2354,12 @@ L<https://docs.google.com/presentation/d/1UkV5oLcTOOXBXPh8foyxko4PR28_zU_aVx6gBm
 
 L<http://leonerds-code.blogspot.co.uk/2013/12/futures-advent-day-1.html>
 
+=item *
+
+"Asynchronous Programming with Futures" - YAPC::EU 2014
+
+L<https://www.youtube.com/watch?v=u9dZgFM6FtE>
+
 =back
 
 =cut
