@@ -1,15 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2016-2020 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2016 -- leonerd@leonerd.org.uk
 
 package Future::Mutex;
 
-use v5.10;
 use strict;
 use warnings;
 
-our $VERSION = '0.48_003';
+our $VERSION = '0.35';
 
 use Future;
 
@@ -19,17 +18,17 @@ C<Future::Mutex> - mutual exclusion lock around code that returns L<Future>s
 
 =head1 SYNOPSIS
 
-   use Future::Mutex;
+ use Future::Mutex;
 
-   my $mutex = Future::Mutex->new;
+ my $mutex = Future::Mutex->new;
 
-   sub do_atomically
-   {
-      return $mutex->enter( sub {
-         ...
-         return $f;
-      });
-   }
+ sub do_atomically
+ {
+    return $mutex->enter( sub {
+       ...
+       return $f;
+    });
+ }
 
 =head1 DESCRIPTION
 
@@ -52,13 +51,6 @@ the first one, until it completes. Once the initial code indicates completion
 (by its returned future providing a result or failing), the next queued code
 is invoked.
 
-An instance may also be a counting mutex if initialised with a count greater
-than one. In this case, it can keep multiple blocks outstanding up to that
-limit, with subsequent requests queued as before. This allows it to act as a
-concurrency-bounding limit around some operation that can run concurrently,
-but an application wishes to apply overall limits to stop it growing too much,
-such as communications with external services or executing other programs.
-
 =cut
 
 =head1 CONSTRUCTOR
@@ -67,32 +59,18 @@ such as communications with external services or executing other programs.
 
 =head2 new
 
-   $mutex = Future::Mutex->new( count => $n )
+   $mutex = Future::Mutex->new
 
 Returns a new C<Future::Mutex> instance. It is initially unlocked.
 
-Takes the following named arguments:
-
-=over 8
-
-=item count => INT
-
-Optional number to limit outstanding concurrency. Will default to 1 if not
-supplied.
-
-=back
-
 =cut
 
 sub new
 {
    my $class = shift;
-   my %params = @_;
 
    return bless {
-      avail => $params{count} // 1,
-      waitf => undef,
-      queue => [],
+      f => Future->done,
    }, $class;
 }
 
@@ -120,43 +98,13 @@ sub enter
    my $self = shift;
    my ( $code ) = @_;
 
-   my $down_f;
-   if( $self->{avail} ) {
-      $self->{avail}--;
-      $down_f = Future->done;
-   }
-   else {
-      die "ARGH Need to clone an existing future\n" unless defined $self->{waitf};
-      push @{ $self->{queue} }, $down_f = $self->{waitf}->new;
-   }
+   my $old_f = $self->{f};
+   $self->{f} = my $new_f = Future->new;
 
-   my $up = sub {
-      if( my $next_f = shift @{ $self->{queue} } ) {
-         $next_f->done;
-      }
-      else {
-         $self->{avail}++;
-         undef $self->{waitf};
-      }
-   };
-
-   my $retf = $down_f->then( $code )->on_ready( $up );
-   $self->{waitf} or $self->{waitf} = $retf;
-   return $retf;
-}
-
-=head2 available
-
-   $avail = $mutex->available
-
-Returns true if the mutex is currently unlocked, or false if it is locked.
-
-=cut
-
-sub available
-{
-   my $self = shift;
-   return $self->{avail};
+   $old_f->then( $code )
+      ->then_with_f(
+         ( sub { my $f = shift; $new_f->done; $f; } ) x 2
+      );
 }
 
 =head1 AUTHOR
