@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2013-2015 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2013-2016 -- leonerd@leonerd.org.uk
 
 package Future::Utils;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.32';
+our $VERSION = '0.35';
 
 use Exporter 'import';
 # Can't import the one from Exporter as it relies on package inheritance
@@ -108,11 +108,18 @@ Z<>
     return $item_f;
  } foreach => \@items, concurrent => 10;
 
+Unless otherwise noted, the following functions require at least version
+I<0.08>.
+
 =cut
 
 =head1 INVOKING A BLOCK OF CODE
 
-=head2 $f = call { CODE }
+=head2 call
+
+   $f = call { CODE }
+
+I<Since version 0.22.>
 
 The C<call> function invokes a block of code that returns a future, and simply
 returns the future it returned. The code is wrapped in an C<eval {}> block, so
@@ -131,7 +138,11 @@ sub call(&)
    return Future->call( $code );
 }
 
-=head2 $f = call_with_escape { CODE }
+=head2 call_with_escape
+
+   $f = call_with_escape { CODE }
+
+I<Since version 0.22.>
 
 The C<call_with_escape> function invokes a block of code that returns a
 future, and passes in a separate future (called here an "escape future").
@@ -181,7 +192,9 @@ If some specific subclass or instance of C<Future> is required as the return
 value, it can be passed as the C<return> argument. Otherwise the return value
 will be constructed by cloning the first non-immediate trial C<Future>.
 
-=head2 $future = repeat { CODE } while => CODE
+=head2 repeat+while
+
+   $future = repeat { CODE } while => CODE
 
 Repeatedly calls the C<CODE> block while the C<while> condition returns a true
 value. Each time the trial future completes, the C<while> condition is passed
@@ -194,7 +207,9 @@ If the C<$code> block dies entirely and throws an exception, this will be
 caught and considered as an immediately-failed C<Future> with the exception as
 the future's failure. The exception will not be propagated to the caller.
 
-=head2 $future = repeat { CODE } until => CODE
+=head2 repeat+until
+
+   $future = repeat { CODE } until => CODE
 
 Repeatedly calls the C<CODE> block until the C<until> condition returns a true
 value. Each time the trial future completes, the C<until> condition is passed
@@ -203,7 +218,11 @@ the trial future.
  $trial_f = $code->( $previous_trial_f )
  $accept = $until->( $trial_f )
 
-=head2 $future = repeat { CODE } foreach => ARRAY, otherwise => CODE
+=head2 repeat+foreach
+
+   $future = repeat { CODE } foreach => ARRAY, otherwise => CODE
+
+I<Since version 0.13.>
 
 Calls the C<CODE> block once for each value obtained from the array, passing
 in the value as the first argument (before the previous trial future). When
@@ -222,9 +241,17 @@ eventual future will simply be that of the last trial. If there was no trial,
 because the C<foreach> list was already empty, then an immediate successful
 future with an empty result is returned.
 
-=head2 $future = repeat { CODE } foreach => ARRAY, while => CODE, ...
+=head2 repeat+foreach+while
 
-=head2 $future = repeat { CODE } foreach => ARRAY, until => CODE, ...
+   $future = repeat { CODE } foreach => ARRAY, while => CODE, ...
+
+I<Since version 0.13.>
+
+=head2 repeat+foreach+until
+
+   $future = repeat { CODE } foreach => ARRAY, until => CODE, ...
+
+I<Since version 0.13.>
 
 Combines the effects of C<foreach> with C<while> or C<until>. Calls the
 C<CODE> block once for each value obtained from the array, until the array is
@@ -236,7 +263,11 @@ operation is terminated early due to the C<while> or C<until> condition being
 satisfied, the eventual result will simply be that of the last trial that was
 executed.
 
-=head2 $future = repeat { CODE } generate => CODE, otherwise => CODE
+=head2 repeat+generate
+
+   $future = repeat { CODE } generate => CODE, otherwise => CODE
+
+I<Since version 0.13.>
 
 Calls the C<CODE> block once for each value obtained from the generator code,
 passing in the value as the first argument (before the previous trial future).
@@ -378,7 +409,11 @@ sub repeat(&@)
    return $future;
 }
 
-=head2 $future = try_repeat { CODE } ...
+=head2 try_repeat
+
+   $future = try_repeat { CODE } ...
+
+I<Since version 0.18.>
 
 A variant of C<repeat> that doesn't warn when the trial fails and the
 condition code asks for it to be repeated.
@@ -397,7 +432,11 @@ sub try_repeat(&@)
    &repeat( @_, try => 1 );
 }
 
-=head2 $future = try_repeat_until_success { CODE } ...
+=head2 try_repeat_until_success
+
+   $future = try_repeat_until_success { CODE } ...
+
+I<Since version 0.18.>
 
 A shortcut to calling C<try_repeat> with an ending condition that simply tests
 for a successful result from a future. May be combined with C<foreach> or
@@ -489,6 +528,9 @@ The expected return value from each item's future, and the value returned from
 the result future will differ in each function's case; they are documented
 below.
 
+For similarity with perl's core C<map> function, the item is also available
+aliased as C<$_>.
+
 =cut
 
 # This function is invoked in two circumstances:
@@ -518,7 +560,7 @@ sub _fmap_slot
             return $return;
          }
 
-         my $f = $slots->[$idx] = Future->call( $code, $item );
+         my $f = $slots->[$idx] = Future->call( $code, local $_ = $item );
 
          if( $collect eq "array" ) {
             push @$results, my $r = [];
@@ -597,13 +639,17 @@ sub _fmap
       !defined $_ or $_->is_ready or $_->cancel for @slots;
    });
    $future->on_cancel( sub {
-      $_->cancel for @slots;
+      !defined $_ or $_->is_ready or $_->cancel for @slots;
    });
 
    return $future;
 }
 
-=head2 $future = fmap_concat { CODE } ...
+=head2 fmap_concat
+
+   $future = fmap_concat { CODE } ...
+
+I<Since version 0.14.>
 
 This version of C<fmap> expects each item future to return a list of zero or
 more values, and the overall result will be the concatenation of all these
@@ -632,7 +678,11 @@ sub fmap_concat(&@)
 }
 *fmap = \&fmap_concat;
 
-=head2 $future = fmap_scalar { CODE } ...
+=head2 fmap_scalar
+
+   $future = fmap_scalar { CODE } ...
+
+I<Since version 0.14.>
 
 This version of C<fmap> acts more like the C<map> functions found in Scheme or
 Haskell; it expects that each item future returns only one value, and the
@@ -654,7 +704,11 @@ sub fmap_scalar(&@)
 }
 *fmap1 = \&fmap_scalar;
 
-=head2 $future = fmap_void { CODE } ...
+=head2 fmap_void
+
+   $future = fmap_void { CODE } ...
+
+I<Since version 0.14.>
 
 This version of C<fmap> does not collect any results from its item futures, it
 simply waits for them all to complete. Its result future will provide no
